@@ -6,6 +6,7 @@ import java.io.Closeable;
 import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.FileWriter;
+import java.io.Flushable;
 import java.io.IOException;
 import java.io.Writer;
 import java.sql.SQLException;
@@ -38,7 +39,7 @@ import com.openexchange.log.LogFactory;
 
 /**
  * This class contains the implementation of the API defined in {@link OXContextRestoreInterface}
- * 
+ *
  * @author <a href="mailto:dennis.sieben@open-xchange.com">Dennis Sieben</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>: Bugfix 20044
  */
@@ -59,6 +60,21 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
         }
     }
 
+    /**
+     * Safely flushes specified {@link Flushable} instance.
+     *
+     * @param toFlush The {@link Flushable} instance
+     */
+    protected static void flush(final Flushable toFlush) {
+        if (null != toFlush) {
+            try {
+                toFlush.flush();
+            } catch (final Exception e) {
+                // Ignore
+            }
+        }
+    }
+
     /**
      * Parser for MySQL dump files.
      */
@@ -69,11 +85,13 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
             private final int poolId;
             private final int contextId;
             private final String schema;
+            private final String fileName;
             private VersionInformation versionInformation;
             private UpdateTaskInformation updateTaskInformation;
 
-            protected PoolIdSchemaAndVersionInfo(final int contextId, int poolId, String schema, VersionInformation versionInformation, UpdateTaskInformation updateTaskInformation) {
+            protected PoolIdSchemaAndVersionInfo(final String fileName, final int contextId, int poolId, String schema, VersionInformation versionInformation, UpdateTaskInformation updateTaskInformation) {
                 super();
+                this.fileName = fileName;
                 this.contextId = contextId;
                 this.poolId = poolId;
                 this.schema = schema;
@@ -81,6 +99,10 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
                 this.updateTaskInformation = updateTaskInformation;
             }
 
+            public String getFileName() {
+                return fileName;
+            }
+
             public int getContextId() {
                 return contextId;
             }
@@ -129,7 +151,7 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
 
         /**
          * Starts parsing named MySQL dump file
-         * 
+         *
          * @param cid The context identifier
          * @param fileName The name of the MySQL dump file
          * @return The information object for parsed MySQL dump file
@@ -184,13 +206,14 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
                                 }
                                 LOG.info("Database: " + databasename);
                                 if (null != bufferedWriter) {
-                                    bufferedWriter.append("/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;" + '\n');
+                                    bufferedWriter.append("/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;\n");
+                                    bufferedWriter.flush();
                                     bufferedWriter.close();
                                 }
 
                                 final String file = "/tmp/" + databasename + ".txt";
                                 bufferedWriter = new BufferedWriter(new FileWriter(file));
-                                bufferedWriter.append("/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;" + '\n');
+                                bufferedWriter.append("/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;\n");
                                 // Reset values
                                 cidpos = -1;
                                 state = 0;
@@ -289,13 +312,14 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
                     }
                 }
             } finally {
+                flush(bufferedWriter);
                 close(bufferedWriter);
                 close(in);
             }
-            if (null == updateTaskInformation) {
-                throw new OXContextRestoreException(Code.NO_UPDATE_TASK_INFORMATION_FOUND);
-            }
-            return new PoolIdSchemaAndVersionInfo(cid, poolId, schema, versionInformation, updateTaskInformation);
+            //if (null == updateTaskInformation) {
+            //    throw new OXContextRestoreException(Code.NO_UPDATE_TASK_INFORMATION_FOUND);
+            // }
+            return new PoolIdSchemaAndVersionInfo(fileName, cid, poolId, schema, versionInformation, updateTaskInformation);
         }
 
         /**
@@ -526,7 +550,7 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
         /**
          * Searches for the cid and returns the line number in which is was found, after this method the reader's position is behind the
          * create structure
-         * 
+         *
          * @param in
          * @return
          * @throws IOException
@@ -589,7 +613,7 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
     }
 
     protected final static Log LOG = LogFactory.getLog(OXContextRestore.class);
-    
+
     private final BasicAuthenticator basicauth;
 
     public OXContextRestore() throws StorageException {
@@ -608,7 +632,7 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
             LOG.error("One of the arguments for restore is null", e);
             throw e;
         }
-        
+
         try {
             basicauth.doAuthentication(auth);
         } catch (final InvalidCredentialsException e) {
@@ -619,11 +643,11 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
         final Parser parser = new Parser();
         LOG.info("Context: " + ctx);
         LOG.info("Filenames: " + java.util.Arrays.toString(fileNames));
-        
+
         try {
             VersionInformation versionInfo = null;
             UpdateTaskInformation updateTaskInfo = null;
-            PoolIdSchemaAndVersionInfo result = null; 
+            PoolIdSchemaAndVersionInfo result = null;
             for (final String fileName : fileNames) {
                 final PoolIdSchemaAndVersionInfo infoObject = parser.start(ctx.getId().intValue(), fileName);
                 final VersionInformation versionInformation = infoObject.getVersionInformation();
@@ -643,14 +667,14 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
             if (null == result) {
                 throw new OXContextRestoreException(Code.NO_CONFIGDB_FOUND);
             }
-            
+
             final OXContextRestoreStorageInterface instance = OXContextRestoreStorageInterface.getInstance();
             result.setVersionInformation(versionInfo);
             result.setUpdateTaskInformation(updateTaskInfo);
             instance.checkVersion(result);
-            
+
             final OXContextInterface contextInterface = Activator.getContextInterface();
-            
+
             final OXToolStorageInterface storage = OXToolStorageInterface.getInstance();
             if (storage.existsContext(ctx)) {
                 try {
@@ -670,13 +694,13 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
             throw e;
         } catch (final FileNotFoundException e) {
             LOG.error(e.getMessage(), e);
-            throw new OXContextRestoreException(Code.FILE_NOT_FOUND);
+            throw new OXContextRestoreException(Code.FILE_NOT_FOUND, e);
         } catch (final IOException e) {
             LOG.error(e.getMessage(), e);
-            throw new OXContextRestoreException(Code.IO_EXCEPTION);
+            throw new OXContextRestoreException(Code.IO_EXCEPTION, e);
         } catch (final SQLException e) {
             LOG.error(e.getMessage(), e);
-            throw new OXContextRestoreException(Code.DATABASE_OPERATION_ERROR, e.getMessage());
+            throw new OXContextRestoreException(Code.DATABASE_OPERATION_ERROR, e, e.getMessage());
         } catch (final OXContextRestoreException e) {
             LOG.error(e.getMessage(), e);
             throw e;
@@ -688,5 +712,5 @@ public class OXContextRestore extends OXCommonImpl implements OXContextRestoreIn
             throw e;
         }
     }
-    
+
 }
